@@ -51,7 +51,7 @@
 #include <linux/gen_stats.h>
 #include <linux/ip_mp_alg.h>
 #include <linux/atm.h>
-#include <linux/ip.h>
+#include <linux/inetdevice.h>
 #include <linux/ipv6.h>
 #include <linux/snmp.h>
 
@@ -67,8 +67,6 @@
 #include <netlink-private/cache-api.h>
 #include <netlink-private/types.h>
 
-#define NSEC_PER_SEC	1000000000L
-
 struct trans_tbl {
 	int i;
 	const char *a;
@@ -82,36 +80,24 @@ struct trans_list {
 	struct nl_list_head list;
 };
 
-#ifdef NL_DEBUG
-#define NL_DBG(LVL,FMT,ARG...)						\
-	do {								\
-		if (LVL <= nl_debug)					\
-			fprintf(stderr,					\
-				"DBG<" #LVL ">%20s:%-4u %s: " FMT,	\
-				__FILE__, __LINE__,			\
-				__PRETTY_FUNCTION__, ##ARG);		\
+#define NL_DEBUG	1
+
+#define NL_DBG(LVL,FMT,ARG...) \
+	do {	\
+		if (LVL <= nl_debug) \
+			fprintf(stderr, "DBG<" #LVL ">: " FMT, ##ARG); \
 	} while (0)
-#else /* NL_DEBUG */
-#define NL_DBG(LVL,FMT,ARG...) do { } while(0)
-#endif /* NL_DEBUG */
 
 #define BUG()                            				\
 	do {                                 				\
-		fprintf(stderr, "BUG at file position %s:%d:%s\n",  	\
-			__FILE__, __LINE__, __PRETTY_FUNCTION__); 	\
+		NL_DBG(1, "BUG: %s:%d\n",  				\
+			__FILE__, __LINE__);         			\
 		assert(0);						\
 	} while (0)
 
-#define BUG_ON(condition)						\
-	do {								\
-		if (condition)						\
-			BUG();						\
-	} while (0)
-
-
 #define APPBUG(msg)							\
 	do {								\
-		fprintf(stderr, "APPLICATION BUG: %s:%d:%s: %s\n",	\
+		NL_DBG(1, "APPLICATION BUG: %s:%d:%s: %s\n",		\
 			__FILE__, __LINE__, __PRETTY_FUNCTION__, msg);	\
 		assert(0);						\
 	} while(0)
@@ -135,12 +121,7 @@ extern void dump_from_ops(struct nl_object *, struct nl_dump_params *);
 
 static inline int nl_cb_call(struct nl_cb *cb, int type, struct nl_msg *msg)
 {
-	int ret;
-
-	cb->cb_active = type;
-	ret = cb->cb_set[type](msg, cb->cb_args[type]);
-	cb->cb_active = __NL_CB_TYPE_MAX;
-	return ret;
+	return cb->cb_set[type](msg, cb->cb_args[type]);
 }
 
 #define ARRAY_SIZE(X) (sizeof(X) / sizeof((X)[0]))
