@@ -7,7 +7,6 @@
  *	of the License.
  *
  * Copyright (c) 2003-2013 Thomas Graf <tgraf@suug.ch>
- * Copyright (c) 2013 Sassano Systems LLC <joe@sassanosystems.com>
  */
 
 #ifndef NETLINK_LOCAL_TYPES_H_
@@ -18,24 +17,13 @@
 #include <netlink/route/qdisc.h>
 #include <netlink/route/rtnl.h>
 #include <netlink/route/route.h>
-#include <netlink/idiag/idiagnl.h>
-#include <netlink/netfilter/ct.h>
-#include <netlink-private/object-api.h>
 #include <netlink-private/route/tc-api.h>
-#include <netlink-private/route/link/sriov.h>
-#include <linux/netlink.h>
-#include <linux/rtnetlink.h>
-#include <linux/genetlink.h>
-#include <linux/tc_act/tc_mirred.h>
-#include <linux/tc_act/tc_skbedit.h>
-#include <linux/tc_act/tc_gact.h>
-#include <linux/sock_diag.h>
 
+#define NL_SOCK_BUFSIZE_SET	(1<<0)
 #define NL_SOCK_PASSCRED	(1<<1)
 #define NL_OWN_PORT		(1<<2)
 #define NL_MSG_PEEK		(1<<3)
-#define NL_MSG_PEEK_EXPLICIT	(1<<4)
-#define NL_NO_AUTO_ACK		(1<<5)
+#define NL_NO_AUTO_ACK		(1<<4)
 
 #define NL_MSG_CRED_PRESENT 1
 
@@ -43,13 +31,12 @@ struct nl_cache_ops;
 struct nl_sock;
 struct nl_object;
 struct nl_hash_table;
-struct nl_vf_vlans;
 
 struct nl_cb
 {
 	nl_recvmsg_msg_cb_t	cb_set[NL_CB_TYPE_MAX+1];
 	void *			cb_args[NL_CB_TYPE_MAX+1];
-
+	
 	nl_recvmsg_err_cb_t	cb_err;
 	void *			cb_err_arg;
 
@@ -71,8 +58,6 @@ struct nl_cb
 					      struct nl_msg *);
 
 	int			cb_refcnt;
-	/** indicates the callback that is currently active */
-	enum nl_cb_type		cb_active;
 };
 
 struct nl_sock
@@ -104,7 +89,6 @@ struct nl_cache_assoc
 {
 	struct nl_cache *	ca_cache;
 	change_func_t		ca_change;
-	change_func_v2_t	ca_change_v2;
 	void *			ca_change_data;
 };
 
@@ -121,7 +105,6 @@ struct nl_cache_mngr
 struct nl_parser_param;
 
 #define LOOSE_COMPARISON	1
-#define ID_COMPARISON           2
 
 #define NL_OBJ_MARK		1
 
@@ -163,26 +146,6 @@ struct rtnl_link_map
 	uint8_t  lm_port;
 };
 
-struct rtnl_link_vf
-{
-	struct nl_list_head	vf_list;
-	int			ce_refcnt;
-	uint32_t		ce_mask;
-	uint32_t		vf_index;
-	uint64_t		vf_guid_node;
-	uint64_t		vf_guid_port;
-	uint32_t		vf_linkstate;
-	struct nl_addr *	vf_lladdr;
-	uint32_t		vf_max_tx_rate;
-	uint32_t		vf_min_tx_rate;
-	uint32_t		vf_rate;
-	uint32_t		vf_rss_query_en;
-	uint32_t		vf_spoofchk;
-	uint64_t		vf_stats[RTNL_LINK_VF_STATS_MAX+1];
-	uint32_t		vf_trust;
-	struct nl_vf_vlans *	vf_vlans;
-};
-
 #define IFQDISCSIZ	32
 
 struct rtnl_link
@@ -195,13 +158,12 @@ struct rtnl_link
 	uint32_t			l_index;
 	uint32_t			l_flags;
 	uint32_t			l_change;
-	uint32_t			l_mtu;
+	uint32_t 			l_mtu;
 	uint32_t			l_link;
-	int32_t                         l_link_netnsid;
 	uint32_t			l_txqlen;
 	uint32_t			l_weight;
 	uint32_t			l_master;
-	struct nl_addr *		l_addr;
+	struct nl_addr *		l_addr;	
 	struct nl_addr *		l_bcast;
 	char				l_qdisc[IFQDISCSIZ];
 	struct rtnl_link_map		l_map;
@@ -219,19 +181,8 @@ struct rtnl_link
 	uint32_t			l_promiscuity;
 	uint32_t			l_num_tx_queues;
 	uint32_t			l_num_rx_queues;
-	uint32_t			l_gso_max_segs;
-	uint32_t			l_gso_max_size;
 	uint32_t			l_group;
 	uint8_t				l_carrier;
-	/* 3 byte hole */
-	uint32_t			l_carrier_changes;
-	struct rtnl_link_af_ops *	l_af_ops;
-	struct nl_data *		l_phys_port_id;
-	char				l_phys_port_name[IFNAMSIZ];
-	struct nl_data *		l_phys_switch_id;
-	int				l_ns_fd;
-	pid_t				l_ns_pid;
-	struct rtnl_link_vf *		l_vf_list;
 };
 
 struct rtnl_ncacheinfo
@@ -250,30 +201,29 @@ struct rtnl_neigh
 	uint32_t	n_ifindex;
 	uint16_t	n_state;
 	uint8_t		n_flags;
-	uint8_t		n_type;
+	uint8_t		n_type;	
 	struct nl_addr *n_lladdr;
-	struct nl_addr *n_dst;
+	struct nl_addr *n_dst;	
 	uint32_t	n_probes;
 	struct rtnl_ncacheinfo n_cacheinfo;
 	uint32_t                n_state_mask;
 	uint32_t                n_flag_mask;
 	uint32_t		n_master;
-	uint16_t	n_vlan;
 };
 
 
 struct rtnl_addr_cacheinfo
 {
-	/* Preferred lifetime in seconds, ticking from when the message gets constructed */
+	/* Preferred lifetime in seconds */
 	uint32_t aci_prefered;
 
-	/* Valid lifetime in seconds, ticking from when the message gets constructed */
+	/* Valid lifetime in seconds */
 	uint32_t aci_valid;
 
-	/* Timestamp of creation in 1/100s since boottime, clock_gettime(CLOCK_MONOTONIC) */
+	/* Timestamp of creation in 1/100s seince boottime */
 	uint32_t aci_cstamp;
 
-	/* Timestamp of last update in 1/100s since boottime, clock_gettime(CLOCK_MONOTONIC) */
+	/* Timestamp of last update in 1/100s since boottime */
 	uint32_t aci_tstamp;
 };
 
@@ -283,18 +233,18 @@ struct rtnl_addr
 
 	uint8_t		a_family;
 	uint8_t		a_prefixlen;
+	uint8_t		a_flags;
 	uint8_t		a_scope;
-	uint32_t	a_flags;
 	uint32_t	a_ifindex;
 
-	struct nl_addr *a_peer;
+	struct nl_addr *a_peer;	
 	struct nl_addr *a_local;
 	struct nl_addr *a_bcast;
 	struct nl_addr *a_anycast;
 	struct nl_addr *a_multicast;
 
 	struct rtnl_addr_cacheinfo a_cacheinfo;
-
+	
 	char a_label[IFNAMSIZ];
 	uint32_t a_flag_mask;
 	struct rtnl_link *a_link;
@@ -446,7 +396,7 @@ struct rtnl_neightbl_parms
 	 * Queue length for the delayed proxy arp requests.
 	 */
 	uint32_t		ntp_proxy_qlen;
-
+	
 	/**
 	 * Mask of available parameter attributes
 	 */
@@ -548,31 +498,6 @@ struct rtnl_cls
 	uint16_t		c_protocol;
 };
 
-struct rtnl_act
-{
-	NL_TC_GENERIC(c);
-	struct rtnl_act *	a_next;
-};
-
-struct rtnl_mirred
-{
-	struct tc_mirred m_parm;
-};
-
-struct rtnl_skbedit
-{
-	struct tc_skbedit s_parm;
-	uint32_t	  s_flags;
-	uint32_t	  s_mark;
-	uint32_t	  s_prio;
-	uint16_t	  s_queue_mapping;
-};
-
-struct rtnl_gact
-{
-	struct tc_gact g_parm;
-};
-
 struct rtnl_u32
 {
 	uint32_t		cu_divisor;
@@ -581,8 +506,7 @@ struct rtnl_u32
 	uint32_t		cu_link;
 	struct nl_data *	cu_pcnt;
 	struct nl_data *	cu_selector;
-	struct nl_data *	cu_mark;
-	struct rtnl_act*	cu_act;
+	struct nl_data *	cu_act;
 	struct nl_data *	cu_police;
 	char			cu_indev[IFNAMSIZ];
 	int			cu_mask;
@@ -764,31 +688,6 @@ struct rtnl_plug
 	uint32_t        limit;
 };
 
-struct rtnl_fq_codel
-{
-	int		fq_limit;
-	uint32_t	fq_target;
-	uint32_t	fq_interval;
-	int		fq_flows;
-	uint32_t	fq_quantum;
-	int		fq_ecn;
-	uint32_t	fq_mask;
-};
-
-struct rtnl_hfsc_qdisc
-{
-	uint32_t		qh_defcls;
-	uint32_t		qh_mask;
-};
-
-struct rtnl_hfsc_class
-{
-	struct tc_service_curve ch_rsc;
-	struct tc_service_curve ch_fsc;
-	struct tc_service_curve ch_usc;
-	uint32_t		ch_mask;
-};
-
 struct flnl_request
 {
 	NLHDR_COMMON
@@ -887,12 +786,9 @@ struct nfnl_ct {
 	uint32_t		ct_mark;
 	uint32_t		ct_use;
 	uint32_t		ct_id;
-	uint16_t		ct_zone;
 
 	struct nfnl_ct_dir	ct_orig;
 	struct nfnl_ct_dir	ct_repl;
-
-	struct nfnl_ct_timestamp ct_tstamp;
 };
 
 union nfnl_exp_protodata {
@@ -1011,280 +907,4 @@ struct ematch_quoted {
 	int	index;
 };
 
-struct idiagnl_meminfo {
-	NLHDR_COMMON
-
-	uint32_t idiag_rmem;
-	uint32_t idiag_wmem;
-	uint32_t idiag_fmem;
-	uint32_t idiag_tmem;
-};
-
-struct idiagnl_vegasinfo {
-	NLHDR_COMMON
-
-	uint32_t tcpv_enabled;
-	uint32_t tcpv_rttcnt;
-	uint32_t tcpv_rtt;
-	uint32_t tcpv_minrtt;
-};
-
-struct idiagnl_msg {
-	NLHDR_COMMON
-
-	uint8_t			    idiag_family;
-	uint8_t			    idiag_state;
-	uint8_t			    idiag_timer;
-	uint8_t			    idiag_retrans;
-	uint16_t		    idiag_sport;
-	uint16_t		    idiag_dport;
-	struct nl_addr *	    idiag_src;
-	struct nl_addr *	    idiag_dst;
-	uint32_t		    idiag_ifindex;
-	uint32_t		    idiag_expires;
-	uint32_t		    idiag_rqueue;
-	uint32_t		    idiag_wqueue;
-	uint32_t		    idiag_uid;
-	uint32_t		    idiag_inode;
-
-	uint8_t			    idiag_tos;
-	uint8_t			    idiag_tclass;
-	uint8_t			    idiag_shutdown;
-	char *			    idiag_cong;
-	struct idiagnl_meminfo *    idiag_meminfo;
-	struct idiagnl_vegasinfo *  idiag_vegasinfo;
-	struct tcp_info		    idiag_tcpinfo;
-	uint32_t		    idiag_skmeminfo[SK_MEMINFO_VARS];
-};
-
-struct idiagnl_req {
-	NLHDR_COMMON
-
-	uint8_t			idiag_family;
-	uint8_t			idiag_ext;
-	struct nl_addr *	idiag_src;
-	struct nl_addr *	idiag_dst;
-	uint32_t		idiag_ifindex;
-	uint32_t		idiag_states;
-	uint32_t		idiag_dbs;
-};
-
-// XFRM related definitions
-
-/* Selector, used as selector both on policy rules (SPD) and SAs. */
-struct xfrmnl_sel {
-	uint32_t        refcnt;
-	struct nl_addr* daddr;
-	struct nl_addr* saddr;
-	uint16_t        dport;
-	uint16_t        dport_mask;
-	uint16_t        sport;
-	uint16_t        sport_mask;
-	uint16_t        family;
-	uint8_t         prefixlen_d;
-	uint8_t         prefixlen_s;
-	uint8_t         proto;
-	int32_t         ifindex;
-	uint32_t        user;
-};
-
-/* Lifetime configuration, used for both policy rules (SPD) and SAs. */
-struct xfrmnl_ltime_cfg {
-	uint32_t        refcnt;
-	uint64_t        soft_byte_limit;
-	uint64_t        hard_byte_limit;
-	uint64_t        soft_packet_limit;
-	uint64_t        hard_packet_limit;
-	uint64_t        soft_add_expires_seconds;
-	uint64_t        hard_add_expires_seconds;
-	uint64_t        soft_use_expires_seconds;
-	uint64_t        hard_use_expires_seconds;
-};
-
-/* Current lifetime, used for both policy rules (SPD) and SAs. */
-struct xfrmnl_lifetime_cur {
-	uint64_t        bytes;
-	uint64_t        packets;
-	uint64_t        add_time;
-	uint64_t        use_time;
-};
-
-struct xfrmnl_replay_state {
-	uint32_t        oseq;
-	uint32_t        seq;
-	uint32_t        bitmap;
-};
-
-struct xfrmnl_replay_state_esn {
-	uint32_t        bmp_len;
-	uint32_t        oseq;
-	uint32_t        seq;
-	uint32_t        oseq_hi;
-	uint32_t        seq_hi;
-	uint32_t        replay_window;
-	uint32_t        bmp[0];
-};
-
-struct xfrmnl_mark {
-	uint32_t        v; /* value */
-	uint32_t        m; /* mask */
-};
-
-/* XFRM AE related definitions */
-
-struct xfrmnl_sa_id {
-	struct nl_addr* daddr;
-	uint32_t        spi;
-	uint16_t        family;
-	uint8_t         proto;
-};
-
-struct xfrmnl_ae {
-	NLHDR_COMMON
-
-	struct xfrmnl_sa_id             sa_id;
-	struct nl_addr*                 saddr;
-	uint32_t                        flags;
-	uint32_t                        reqid;
-	struct xfrmnl_mark              mark;
-	struct xfrmnl_lifetime_cur      lifetime_cur;
-	uint32_t                        replay_maxage;
-	uint32_t                        replay_maxdiff;
-	struct xfrmnl_replay_state      replay_state;
-	struct xfrmnl_replay_state_esn* replay_state_esn;
-};
-
-/* XFRM SA related definitions */
-
-struct xfrmnl_id {
-	struct nl_addr* daddr;
-	uint32_t        spi;
-	uint8_t         proto;
-};
-
-struct xfrmnl_stats {
-	uint32_t        replay_window;
-	uint32_t        replay;
-	uint32_t        integrity_failed;
-};
-
-struct xfrmnl_algo_aead {
-	char            alg_name[64];
-	uint32_t        alg_key_len;    /* in bits */
-	uint32_t        alg_icv_len;    /* in bits */
-	char            alg_key[0];
-};
-
-struct xfrmnl_algo_auth {
-	char            alg_name[64];
-	uint32_t        alg_key_len;    /* in bits */
-	uint32_t        alg_trunc_len;  /* in bits */
-	char            alg_key[0];
-};
-
-struct xfrmnl_algo {
-	char            alg_name[64];
-	uint32_t        alg_key_len;    /* in bits */
-	char            alg_key[0];
-};
-
-struct xfrmnl_encap_tmpl {
-	uint16_t        encap_type;
-	uint16_t        encap_sport;
-	uint16_t        encap_dport;
-	struct nl_addr* encap_oa;
-};
-
-struct xfrmnl_sa {
-	NLHDR_COMMON
-
-	struct xfrmnl_sel*              sel;
-	struct xfrmnl_id                id;
-	struct nl_addr*                 saddr;
-	struct xfrmnl_ltime_cfg*        lft;
-	struct xfrmnl_lifetime_cur      curlft;
-	struct xfrmnl_stats             stats;
-	uint32_t                        seq;
-	uint32_t                        reqid;
-	uint16_t                        family;
-	uint8_t                         mode;        /* XFRM_MODE_xxx */
-	uint8_t                         replay_window;
-	uint8_t                         flags;
-	struct xfrmnl_algo_aead*        aead;
-	struct xfrmnl_algo_auth*        auth;
-	struct xfrmnl_algo*             crypt;
-	struct xfrmnl_algo*             comp;
-	struct xfrmnl_encap_tmpl*       encap;
-	uint32_t                        tfcpad;
-	struct nl_addr*                 coaddr;
-	struct xfrmnl_mark              mark;
-	struct xfrmnl_user_sec_ctx*     sec_ctx;
-	uint32_t                        replay_maxage;
-	uint32_t                        replay_maxdiff;
-	struct xfrmnl_replay_state      replay_state;
-	struct xfrmnl_replay_state_esn* replay_state_esn;
-	uint8_t                         hard;
-};
-
-struct xfrmnl_usersa_flush {
-	uint8_t                         proto;
-};
-
-
-/* XFRM SP related definitions */
-
-struct xfrmnl_userpolicy_id {
-	struct xfrmnl_sel               sel;
-	uint32_t                        index;
-	uint8_t                         dir;
-};
-
-struct xfrmnl_user_sec_ctx {
-	uint16_t                        len;
-	uint16_t                        exttype;
-	uint8_t                         ctx_alg;
-	uint8_t                         ctx_doi;
-	uint16_t                        ctx_len;
-	char                            ctx[0];
-};
-
-struct xfrmnl_userpolicy_type {
-	uint8_t                         type;
-	uint16_t                        reserved1;
-	uint16_t                        reserved2;
-};
-
-struct xfrmnl_user_tmpl {
-	struct xfrmnl_id                id;
-	uint16_t                        family;
-	struct nl_addr*                 saddr;
-	uint32_t                        reqid;
-	uint8_t                         mode;
-	uint8_t                         share;
-	uint8_t                         optional;
-	uint32_t                        aalgos;
-	uint32_t                        ealgos;
-	uint32_t                        calgos;
-	struct nl_list_head             utmpl_list;
-};
-
-struct xfrmnl_sp {
-	NLHDR_COMMON
-
-	struct xfrmnl_sel*              sel;
-	struct xfrmnl_ltime_cfg*        lft;
-	struct xfrmnl_lifetime_cur      curlft;
-	uint32_t                        priority;
-	uint32_t                        index;
-	uint8_t                         dir;
-	uint8_t                         action;
-	uint8_t                         flags;
-	uint8_t                         share;
-	struct xfrmnl_user_sec_ctx*     sec_ctx;
-	struct xfrmnl_userpolicy_type   uptype;
-	uint32_t                        nr_user_tmpl;
-	struct nl_list_head             usertmpl_list;
-	struct xfrmnl_mark              mark;
-};
-
 #endif
