@@ -25,18 +25,21 @@ extern "C" {
 /**
  * @ingroup msg
  * @brief
- * Will cause the netlink pid to be set to the pid assigned to
- * the netlink handle (socket) just before sending the message off.
- * @note Requires the use of nl_send_auto_complete()!
+ * Will cause the netlink port to be set to the port assigned to
+ * the netlink icoket ust before sending the message off.
+ *
+ * @note Requires the use of nl_send_auto()!
  */
-#define NL_AUTO_PID	0
+#define NL_AUTO_PORT	0
+#define NL_AUTO_PID	NL_AUTO_PORT
 
 /**
  * @ingroup msg
  * @brief
  * May be used to refer to a sequence number which should be
  * automatically set just before sending the message off.
- * @note Requires the use of nl_send_auto_complete()!
+ *
+ * @note Requires the use of nl_send_auto()!
  */
 #define NL_AUTO_SEQ	0
 
@@ -44,15 +47,13 @@ struct nl_msg;
 struct nl_tree;
 struct ucred;
 
-/* size calculations */
-extern int		  nlmsg_msg_size(int);
-extern int		  nlmsg_total_size(int);
-extern int		  nlmsg_padlen(int);
+extern int			nlmsg_size(int);
+extern int			nlmsg_total_size(int);
+extern int			nlmsg_padlen(int);
 
-/* payload access */
-extern void *		  nlmsg_data(const struct nlmsghdr *);
-extern int		  nlmsg_len(const struct nlmsghdr *);
-extern void *		  nlmsg_tail(const struct nlmsghdr *);
+extern void *			nlmsg_data(const struct nlmsghdr *);
+extern int			nlmsg_datalen(const struct nlmsghdr *);
+extern void *			nlmsg_tail(const struct nlmsghdr *);
 
 /* attribute access */
 extern struct nlattr *	  nlmsg_attrdata(const struct nlmsghdr *, int);
@@ -68,11 +69,6 @@ extern struct nlattr *	  nlmsg_find_attr(struct nlmsghdr *, int, int);
 extern int		  nlmsg_validate(struct nlmsghdr *, int, int,
 					 struct nla_policy *);
 
-/* Backward compatibility */
-#define nlmsg_new()	   		nlmsg_alloc()
-#define nlmsg_build_simple(a, b)	nlmsg_alloc_simple(a, b)
-#define nlmsg_build(ptr)		nlmsg_inherit(ptr)
-
 extern struct nl_msg *	  nlmsg_alloc(void);
 extern struct nl_msg *	  nlmsg_alloc_size(size_t);
 extern struct nl_msg *	  nlmsg_alloc_simple(int, int);
@@ -86,6 +82,7 @@ extern int		  nlmsg_expand(struct nl_msg *, size_t);
 extern struct nlmsghdr *  nlmsg_put(struct nl_msg *, uint32_t, uint32_t,
 				    int, int, int);
 extern struct nlmsghdr *  nlmsg_hdr(struct nl_msg *);
+extern void		  nlmsg_get(struct nl_msg *);
 extern void		  nlmsg_free(struct nl_msg *);
 
 /* attribute modification */
@@ -132,12 +129,14 @@ extern void		nl_msg_dump(struct nl_msg *, FILE *);
  * @arg pos	loop counter, set to current message
  * @arg head	head of message stream
  * @arg len	length of message stream
- * @arg rem	initialized to len, holds bytes currently remaining in stream
  */
+#define nlmsg_for_each(pos, head, len) \
+	for (int rem = len, pos = head; \
+		nlmsg_ok(pos, rem); \
+		pos = nlmsg_next(pos, &rem))
+
 #define nlmsg_for_each_msg(pos, head, len, rem) \
-	for (pos = head, rem = len; \
-	     nlmsg_ok(pos, rem); \
-	     pos = nlmsg_next(pos, &(rem)))
+		nlmsg_for_each(pos, head, len)
 
 /** @} */
 
