@@ -6,7 +6,7 @@
  *	License as published by the Free Software Foundation version 2.1
  *	of the License.
  *
- * Copyright (c) 2003-2011 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2003-2013 Thomas Graf <tgraf@suug.ch>
  */
 
 /**
@@ -15,10 +15,10 @@
  * @{
  */
 
-#include <netlink-local.h>
-#include <netlink-tc.h>
+#include <netlink-private/netlink.h>
+#include <netlink-private/tc.h>
 #include <netlink/netlink.h>
-#include <netlink/route/tc-api.h>
+#include <netlink-private/route/tc-api.h>
 #include <netlink/route/class.h>
 #include <netlink/route/qdisc.h>
 #include <netlink/route/classifier.h>
@@ -100,7 +100,7 @@ void rtnl_class_put(struct rtnl_class *class)
 static int class_build(struct rtnl_class *class, int type, int flags,
 		       struct nl_msg **result)
 {
-	int needed = TCA_ATTR_PARENT | TCA_ATTR_HANDLE;
+	uint32_t needed = TCA_ATTR_PARENT | TCA_ATTR_HANDLE;
 
 	if ((class->ce_mask & needed) == needed &&
 	    TC_H_MAJ(class->c_parent) && TC_H_MAJ(class->c_handle) &&
@@ -196,7 +196,7 @@ int rtnl_class_build_delete_request(struct rtnl_class *class, struct nl_msg **re
 {
 	struct nl_msg *msg;
 	struct tcmsg tchdr;
-	int required = TCA_ATTR_IFINDEX | TCA_ATTR_HANDLE;
+	uint32_t required = TCA_ATTR_IFINDEX | TCA_ATTR_HANDLE;
 
 	if ((class->ce_mask & required) != required) {
 		APPBUG("ifindex and handle must be specified");
