@@ -27,7 +27,6 @@
 #include <netlink/netlink.h>
 #include <netlink-private/route/tc-api.h>
 #include <netlink/route/classifier.h>
-#include <netlink/route/action.h>
 #include <netlink/route/cls/basic.h>
 #include <netlink/route/cls/ematch.h>
 
@@ -36,13 +35,11 @@ struct rtnl_basic
 	uint32_t			b_target;
 	struct rtnl_ematch_tree *	b_ematch;
 	int				b_mask;
-	struct rtnl_act *		b_act;
 };
 
 /** @cond SKIP */
 #define BASIC_ATTR_TARGET	0x001
 #define BASIC_ATTR_EMATCH	0x002
-#define BASIC_ATTR_ACTION	0x004
 /** @endcond */
 
 static struct nla_policy basic_policy[TCA_BASIC_MAX+1] = {
@@ -62,8 +59,6 @@ static void basic_free_data(struct rtnl_tc *tc, void *data)
 	if (!b)
 		return;
 
-	if (b->b_act)
-		rtnl_act_put_all(&b->b_act);
 	rtnl_ematch_tree_free(b->b_ematch);
 }
 
@@ -90,12 +85,6 @@ static int basic_msg_parser(struct rtnl_tc *tc, void *data)
 		if (b->b_ematch)
 			b->b_mask |= BASIC_ATTR_EMATCH;
 	}
-	if (tb[TCA_BASIC_ACT]) {
-		b->b_mask |= BASIC_ATTR_ACTION;
-		err = rtnl_act_parse(&b->b_act, tb[TCA_BASIC_ACT]);
-		if (err)
-			return err;
-	}
 
 	return 0;
 }
@@ -142,21 +131,15 @@ static int basic_msg_fill(struct rtnl_tc *tc, void *data,
 	if (!b)
 		return 0;
 
-	if (b->b_mask & BASIC_ATTR_TARGET)
-		NLA_PUT_U32(msg, TCA_BASIC_CLASSID, b->b_target);
+	if (!(b->b_mask & BASIC_ATTR_TARGET))
+		return -NLE_MISSING_ATTR;
+
+	NLA_PUT_U32(msg, TCA_BASIC_CLASSID, b->b_target);
 
 	if (b->b_mask & BASIC_ATTR_EMATCH &&
 	    rtnl_ematch_fill_attr(msg, TCA_BASIC_EMATCHES, b->b_ematch) < 0)
 		goto nla_put_failure;
-
-	if (b->b_mask & BASIC_ATTR_ACTION) {
-		int err;
-
-		err = rtnl_act_fill(msg, TCA_BASIC_ACT, b->b_act);
-		if (err)
-			return err;
-	}
-
+	
 	return 0;
 
 nla_put_failure:
@@ -217,44 +200,6 @@ struct rtnl_ematch_tree *rtnl_basic_get_ematch(struct rtnl_cls *cls)
 	return b->b_ematch;
 }
 
-int rtnl_basic_add_action(struct rtnl_cls *cls, struct rtnl_act *act)
-{
-	struct rtnl_basic *b;
-
-	if (!act)
-		return 0;
-
-	if (!(b = rtnl_tc_data(TC_CAST(cls))))
-		return -NLE_NOMEM;
-
-	b->b_mask |= BASIC_ATTR_ACTION;
-	/* In case user frees it */
-	rtnl_act_get(act);
-	return rtnl_act_append(&b->b_act, act);
-}
-
-int rtnl_basic_del_action(struct rtnl_cls *cls, struct rtnl_act *act)
-{
-	struct rtnl_basic *b;
-	int ret;
-
-	if (!act)
-		return 0;
-
-	if (!(b = rtnl_tc_data(TC_CAST(cls))))
-		return -NLE_NOMEM;
-
-	if (!(b->b_mask & BASIC_ATTR_ACTION))
-		return -NLE_INVAL;
-	ret = rtnl_act_remove(&b->b_act, act);
-	if (ret)
-		return ret;
-
-	if (!b->b_act)
-		b->b_mask &= ~BASIC_ATTR_ACTION;
-	rtnl_act_put(act);
-	return 0;
-}
 /** @} */
 
 static struct rtnl_tc_ops basic_ops = {
