@@ -20,9 +20,9 @@
  * @{
  */
 
-#include <netlink-local.h>
+#include <netlink-private/netlink.h>
 #include <netlink/netlink.h>
-#include <netlink/route/link/api.h>
+#include <netlink-private/route/link/api.h>
 
 /**
  * Create a new kernel bonding device
@@ -60,15 +60,12 @@ int rtnl_link_bond_add(struct nl_sock *sock, const char *name,
 	if (!name) {
 		if (opts)
 			name = rtnl_link_get_name(opts);
-
-		if (!name)
-			return -NLE_MISSING_ATTR;
 	}
 
 	if ((err = rtnl_link_set_type(link, "bond")) < 0)
 		goto errout;
-	
 
+	if (name)
 		rtnl_link_set_name(link, name);
 
 	err = rtnl_link_add(sock, link, NLM_F_CREATE);
